#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <SDL2/SDL.h>

#include <ball.h>
#include <game.h>
#include <player.h>
#include <pong_ai.h>
#include <render.h>

static void check_events(struct game_data* data, float dt)
{
    SDL_Event event;
    while (SDL_PollEvent(&event))
    {
        switch (event.type)
        {
        case SDL_QUIT:
            printf("Exit requested\n");
            data->is_running = 0;
            break;
        case SDL_KEYDOWN:
        }
    }

    const uint8_t* state = SDL_GetKeyboardState(NULL);
    if (state[SDL_SCANCODE_UP])
    {
        player_move(data->player1, dt, PLAYER_MOVE_DOWN);
    }
    else if (state[SDL_SCANCODE_DOWN])
    {
        player_move(data->player1, dt, PLAYER_MOVE_UP);
    }
}

static void game_update(struct game_data* data, float dt)
{
    // Check and act on events
    check_events(data, dt);

    // Update the AI
    pong_ai_update(data, data->player2, dt);

    // Update all players and the ball
    player_update(data, data->player1, dt);
    player_update(data, data->player2, dt);
    ball_update(data, data->ball, dt);
}

static void game_loop(struct game_data* data)
{
    printf("Starting game loop\n");

    float dt = 0;
    while (data->is_running)
    {
        unsigned long long start = SDL_GetPerformanceCounter();

        // Update the game and render
        game_update(data, dt);
        render(data);

        // Calculate the frame time
        unsigned long long end = SDL_GetPerformanceCounter();
        dt = (end - start) / (float)SDL_GetPerformanceFrequency();

        // Keep the framrate at 60 Fps
        long long delay = 16.666f - (dt * 1000.0f);
        if (delay > 0)
            SDL_Delay(delay);
    }

    printf("Goodbye\n");
}

void game_start(int window_width, int window_height)
{
    printf("Starting game\n");

    // Create a structure to hold all the information about the game
    struct game_data* data = malloc(sizeof(struct game_data));
    memset(data, 0, sizeof(struct game_data));

    // Fill the game data structure with the correct values
    data->is_running = 1;
    data->window_width = window_width;
    data->window_height = window_height;

    // Initialize SDL2
    data->render_data = render_init(window_width, window_height);

    // Create the players and the ball
    data->player1 = player_init(15, window_height, PLAYER_SPEED);
    data->player2 = player_init(window_width - PLAYER_WIDTH - 15, window_height,
                                PLAYER_SPEED);
    data->ball =
        ball_init(window_width / 2 - BALL_WIDTH,
                  window_height / 2 - BALL_WIDTH, BALL_WIDTH, BALL_SPEED);

    game_loop(data);
}